@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim:et sts=4 sw=4
 #
 # ibus-typing-booster - A completion input method for IBus
@@ -94,150 +93,159 @@ class TabSqliteDb:
             'user_freq',
             'timestamp']
 
-        self.old_phrases = []
+        self._old_phrases: List[Tuple[str, str, int]] = []
 
         self.hunspell_obj = hunspell_suggest.Hunspell(())
 
-        if self.user_db_file != ':memory:':
-            if not os.path.exists(self.user_db_file):
-                LOGGER.info(
-                    'The user database %(udb)s does not exist yet.',
-                    {'udb': self.user_db_file})
-            else:
-                try:
-                    desc = self.get_database_desc(self.user_db_file)
-                    if (desc is None
-                            or desc["version"] != USER_DATABASE_VERSION
-                            or (self.get_number_of_columns_of_phrase_table(
-                                self.user_db_file)
-                                != len(self._phrase_table_column_names))):
-                        LOGGER.info(
-                            'The user database %(udb)s seems incompatible',
-                            {'udb': self.user_db_file})
-                        if desc is None:
-                            LOGGER.info(
-                                'No version information in the database')
-                        elif desc["version"] != USER_DATABASE_VERSION:
-                            LOGGER.info(
-                                'The version of the database does not match '
-                                '(too old or too new?)')
-                            LOGGER.info(
-                                'ibus-typing-booster wants version=%s',
-                                USER_DATABASE_VERSION)
-                            LOGGER.info(
-                                'But the  database actually has version=%s',
-                                desc["version"])
-                        elif (self.get_number_of_columns_of_phrase_table(
-                                self.user_db_file)
-                              != len(self._phrase_table_column_names)):
-                            LOGGER.info(
-                                'The number of columns of the database '
-                                'does not match')
-                            LOGGER.info(
-                                'ibus-typing-booster expects %(col)s columns',
-                                {'col': len(self._phrase_table_column_names)})
-                            LOGGER.info(
-                                'The database actually has %(col)s columns',
-                                {'col':
-                                 self.get_number_of_columns_of_phrase_table(
-                                     self.user_db_file)})
-                        LOGGER.info(
-                            'Trying to recover the phrases from the old, '
-                            'incompatible database')
-                        self.old_phrases = self.extract_user_phrases()
-                        timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
-                        LOGGER.info(
-                            'Renaming the incompatible database to "%(name)s"',
-                            {'name': self.user_db_file+timestamp})
-                        if os.path.exists(self.user_db_file):
-                            os.rename(self.user_db_file,
-                                      self.user_db_file+timestamp)
-                        if os.path.exists(self.user_db_file+'-shm'):
-                            os.rename(self.user_db_file+'-shm',
-                                      self.user_db_file+'-shm'+timestamp)
-                        if os.path.exists(self.user_db_file+'-wal'):
-                            os.rename(self.user_db_file+'-wal',
-                                      self.user_db_file+'-wal'+timestamp)
-                        LOGGER.info(
-                            'Creating a new, empty database "%(name)s".',
-                            {'name': self.user_db_file})
-                        self.init_user_db()
-                        LOGGER.info(
-                            'If user phrases were successfully recovered '
-                            'from the old, '
-                            'incompatible database, they will be used to '
-                            'initialize the new database.')
-                    else:
-                        LOGGER.info(
-                            'Compatible database %(db)s found.',
-                            {'db': self.user_db_file})
-                except Exception as error: # pylint: disable=broad-except
-                    LOGGER.exception(
-                        'Unexpected error searching user database: %s: %s',
-                        error.__class__.__name__, error)
+        self._check_database_compatibility()
+        self._check_database_readability()
 
-        # open user phrase database
+        LOGGER.info(
+            'Connect to the database %s.', self.user_db_file)
+        self.database = self.sqlite3_connect_database(self.user_db_file)
         try:
-            LOGGER.info(
-                'Connect to the database %(name)s.',
-                {'name': self.user_db_file})
-            self.database = sqlite3.connect(self.user_db_file)
-            self.set_database_pragma_options()
             self.database.executescript(
                 f'ATTACH DATABASE "{self.user_db_file}" AS user_db;')
         except Exception as error: # pylint: disable=broad-except
             LOGGER.exception(
-                'Could not open the database %s. %s: %s',
+                'Could not attach the database %s. %s: %s '
+                'This should never happen.',
                 self.user_db_file, error.__class__.__name__, error)
-            timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
-            LOGGER.info(
-                'Renaming the incompatible database to "%(name)s".',
-                {'name': self.user_db_file+timestamp})
-            if os.path.exists(self.user_db_file):
-                os.rename(self.user_db_file, self.user_db_file+timestamp)
-            if os.path.exists(self.user_db_file+'-shm'):
-                os.rename(self.user_db_file+'-shm',
-                          self.user_db_file+'-shm'+timestamp)
-            if os.path.exists(self.user_db_file+'-wal'):
-                os.rename(self.user_db_file+'-wal',
-                          self.user_db_file+'-wal'+timestamp)
-            LOGGER.info(
-                'Creating a new, empty database "%(name)s".',
-                {'name': self.user_db_file})
-            self.init_user_db()
-            self.database = sqlite3.connect(self.user_db_file)
-            self.set_database_pragma_options()
-            self.database.executescript(
-                f'ATTACH DATABASE "{self.user_db_file} AS user_db;')
         self.create_tables()
-        if self.old_phrases:
-            sqlargs = []
-            for ophrase in self.old_phrases:
-                sqlargs.append(
-                    {'input_phrase': ophrase[0],
-                     'phrase': ophrase[0],
-                     'p_phrase': '',
-                     'pp_phrase': '',
-                     'user_freq': ophrase[1],
-                     'timestamp': time.time()})
-            sqlstr = '''
-            INSERT INTO user_db.phrases (input_phrase, phrase, p_phrase, pp_phrase, user_freq, timestamp)
-            VALUES (:input_phrase, :phrase, :p_phrase, :pp_phrase, :user_freq, :timestamp)
-            ;'''
-            try:
-                self.database.executemany(sqlstr, sqlargs)
-            except Exception as error: # pylint: disable=broad-except
-                LOGGER.exception(
-                    'Unexpected error inserting old phrases '
-                    'into the user database: %s: %s',
-                    error.__class__.__name__, error)
-            self.database.commit()
-            self.database.execute('PRAGMA wal_checkpoint;')
+        self._restore_old_phrases()
 
         # try create all hunspell-tables in user database
         self.create_indexes(commit=False)
         self.generate_userdb_desc()
 
+    def _restore_old_phrases(self) -> None:
+        '''Restore phrases recovered from old database into new database'''
+        if not self._old_phrases:
+            return
+        LOGGER.info('Restoring old phrases: %s', self._old_phrases)
+        sqlargs = []
+        for ophrase in self._old_phrases:
+            sqlargs.append(
+                {'input_phrase': ophrase[0],
+                 'phrase': ophrase[1],
+                 'p_phrase': '',
+                 'pp_phrase': '',
+                 'user_freq': ophrase[2],
+                 'timestamp': time.time()})
+        sqlstr = '''
+        INSERT INTO user_db.phrases (input_phrase, phrase, p_phrase, pp_phrase, user_freq, timestamp)
+        VALUES (:input_phrase, :phrase, :p_phrase, :pp_phrase, :user_freq, :timestamp)
+        ;'''
+        try:
+            self.database.executemany(sqlstr, sqlargs)
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error inserting old phrases '
+                'into the user database: %s: %s',
+                error.__class__.__name__, error)
+        self.database.commit()
+        self.database.execute('PRAGMA wal_checkpoint;')
+
+    def _check_database_compatibility(self) -> None:
+        '''Check whether the database is compatible'''
+        if self.user_db_file == ':memory:':
+            LOGGER.info(
+                ':memory: database is always newly created '
+                ' and therefore always compatible.')
+            return
+        if not os.path.exists(self.user_db_file):
+            LOGGER.info(
+                'User database %s does not exist yet and will be created. '
+                'A newly created database is always compatible.',
+                self.user_db_file)
+            return
+        try:
+            desc = self.get_database_desc(self.user_db_file)
+            if (desc
+                and
+                desc['version'] == USER_DATABASE_VERSION
+                and
+                self.get_number_of_columns_of_phrase_table(self.user_db_file)
+                == len(self._phrase_table_column_names)):
+                LOGGER.info(
+                    'Compatible database %s found.', self.user_db_file)
+                return
+            LOGGER.info('User database %s seems incompatible.',
+                        self.user_db_file)
+            # Log reason for incompatibility
+            if not desc:
+                LOGGER.info('No version information in the database')
+            elif desc['version'] != USER_DATABASE_VERSION:
+                LOGGER.info('The version of the database does not match '
+                            '(too old or too new?).'
+                            'ibus-typing-booster wants version=%s '
+                            'but the database actually has version=%s.',
+                            USER_DATABASE_VERSION, desc['version'])
+            elif (self.get_number_of_columns_of_phrase_table(self.user_db_file)
+                  != len(self._phrase_table_column_names)):
+                LOGGER.info(
+                    'The number of columns of the database does not match.'
+                    'ibus-typing-booster expectes %s columns but the '
+                    'database actually has %s columns.',
+                    len(self._phrase_table_column_names),
+                    self.get_number_of_columns_of_phrase_table(self.user_db_file))
+            # Try to recover old phrases to use for initializing a new
+            # database:
+            self._old_phrases = self._extract_user_phrases()
+            self._rename_incompatible_or_broken_database()
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error checking database compatibility: %s: %s',
+                error.__class__.__name__, error)
+
+    def _check_database_readability(self) -> None:
+        '''Check whether all rows from the phrases table of the
+        database are readable
+        '''
+        if self.user_db_file == ':memory:':
+            LOGGER.info(
+                ':memory: database is always newly created '
+                ' and therefore always readable.')
+            return
+        if not os.path.exists(self.user_db_file):
+            LOGGER.info(
+                'User database %s does not exist yet and will be created. '
+                'A newly created database is always readable.',
+                self.user_db_file)
+            return
+        database = self.sqlite3_connect_database(self.user_db_file)
+        try:
+            database.execute('SELECT * FROM phrases;').fetchall()
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Error checking database readability: %s: %s',
+                error.__class__.__name__, error)
+            self._rename_incompatible_or_broken_database()
+            return
+        LOGGER.info('Database seems readable.')
+
+    def _rename_incompatible_or_broken_database(self) -> None:
+        '''
+        Creates a backup copy of an incompatible or broken database
+
+        Also creates a new empty database.
+        '''
+        timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
+        LOGGER.info('Renaming incompatible or broken database to "%s"',
+                    self.user_db_file+timestamp)
+        if os.path.exists(self.user_db_file):
+            os.rename(self.user_db_file,
+                      self.user_db_file+timestamp)
+        if os.path.exists(self.user_db_file+'-shm'):
+            os.rename(self.user_db_file+'-shm',
+                      self.user_db_file+'-shm'+timestamp)
+        if os.path.exists(self.user_db_file+'-wal'):
+            os.rename(self.user_db_file+'-wal',
+                      self.user_db_file+'-wal'+timestamp)
+        LOGGER.info(
+            'Creating a new, empty database "%s".', self.user_db_file)
+        self.database = self.sqlite3_connect_database(self.user_db_file)
+
     def update_phrase(
             self,
             input_phrase: str = '',
@@ -281,15 +289,16 @@ class TabSqliteDb:
                 error.__class__.__name__, error)
 
     def sync_usrdb(self) -> None:
-        '''
-        Trigger a checkpoint operation.
-        '''
-        if DEBUG_LEVEL > 1:
-            LOGGER.debug('commit and execute checkpoint ...')
-        self.database.commit()
-        self.database.execute('PRAGMA wal_checkpoint;')
-        if DEBUG_LEVEL > 1:
-            LOGGER.debug('commit and execute checkpoint done.')
+        '''Trigger a checkpoint operation.'''
+        LOGGER.info('commit and execute checkpoint ...')
+        try:
+            self.database.commit()
+            self.database.execute('PRAGMA wal_checkpoint;')
+            LOGGER.info('commit and execute checkpoint done.')
+        except sqlite3.OperationalError as error:
+            LOGGER.exception(
+                'Unexpected error syncing user database: %s: %s',
+                error.__class__.__name__, error)
 
     def create_tables(self) -> None:
         '''Create table for the phrases.'''
@@ -313,10 +322,8 @@ class TabSqliteDb:
         '''
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
-                'input_phrase=%s phrase=%s user_freq=%s ',
-                input_phrase.encode('UTF-8'),
-                phrase.encode('UTF-8'),
-                user_freq)
+                'input_phrase=%r phrase=%r user_freq=%s ',
+                input_phrase, phrase, user_freq)
         if not input_phrase or not phrase:
             return
         input_phrase = itb_util.remove_accents(input_phrase.lower())
@@ -441,6 +448,65 @@ class TabSqliteDb:
                 'best_shortcut_candidates=%s', best_shortcut_candidates)
         return best_shortcut_candidates
 
+    def select_words_empty_input(
+            self,
+            p_phrase: str,
+            pp_phrase: str) -> List[Tuple[str, float]]:
+        '''Get phrases from database which occured previously with
+        any input after the given context
+
+        Returns a list of matches where each match is a tuple in the
+        form of (phrase, user_freq), i.e. returns something like
+        [(phrase, user_freq), ...]
+
+        '''
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug(
+                'p_phrase=%s pp_phrase=%s', p_phrase, pp_phrase)
+        phrase_frequencies: Dict[str, float] = {}
+        if not p_phrase or not pp_phrase:
+            return self.best_candidates(phrase_frequencies)
+        p_phrase = itb_util.remove_accents(p_phrase.lower())
+        pp_phrase = itb_util.remove_accents(pp_phrase.lower())
+        sqlargs = {'p_phrase': p_phrase, 'pp_phrase': pp_phrase}
+        sqlstr = ('SELECT phrase, sum(user_freq) FROM user_db.phrases '
+                  'WHERE p_phrase = :p_phrase '
+                  'AND pp_phrase = :pp_phrase GROUP BY phrase;')
+        results = None
+        try:
+            results = self.database.execute(sqlstr, sqlargs).fetchall()
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error getting phrases for empty input '
+                'with given context from user_db: %s: %s',
+                error.__class__.__name__, error)
+            results = None
+        if not results:
+            return self.best_candidates(phrase_frequencies)
+        sqlstr = (
+            'SELECT sum(user_freq) FROM user_db.phrases '
+            'WHERE p_phrase = :p_phrase AND pp_phrase = :pp_phrase;')
+        count_pp_phrase_p_phrase = 0
+        try:
+            count_pp_phrase_p_phrase = self.database.execute(
+                sqlstr, sqlargs).fetchall()[0][0]
+        except Exception as error: # pylint: disable=broad-except
+            LOGGER.exception(
+                'Unexpected error getting total count for empty input '
+                'with given context from user_db: %s: %s',
+                 error.__class__.__name__, error)
+            count_pp_phrase_p_phrase = 0
+        if not count_pp_phrase_p_phrase:
+            return self.best_candidates(phrase_frequencies)
+        for result in results:
+            phrase_frequencies.update(
+                [(result[0], result[1]/float(count_pp_phrase_p_phrase))])
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug(
+                'Best candidates for empty input with given context=%s',
+                self.best_candidates(phrase_frequencies))
+        return self.best_candidates(phrase_frequencies)
+
     def select_words(
             self,
             input_phrase: str,
@@ -453,18 +519,19 @@ class TabSqliteDb:
         form of (phrase, user_freq), i.e. returns something like
         [(phrase, user_freq), ...]
         '''
-        if input_phrase:
-            input_phrase = unicodedata.normalize(
-                itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
-        p_phrase = itb_util.remove_accents(p_phrase.lower())
-        pp_phrase = itb_util.remove_accents(pp_phrase.lower())
-        title_case = input_phrase.istitle()
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'input_phrase=%s p_phrase=%s pp_phrase=%s',
                 input_phrase, p_phrase, pp_phrase)
+        if not input_phrase:
+            return self.select_words_empty_input(p_phrase, pp_phrase)
         phrase_frequencies: Dict[str, float] = {}
-        if input_phrase and not ' ' in input_phrase:
+        input_phrase = unicodedata.normalize(
+            itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
+        p_phrase = itb_util.remove_accents(p_phrase.lower())
+        pp_phrase = itb_util.remove_accents(pp_phrase.lower())
+        title_case = input_phrase.istitle()
+        if ' ' not in input_phrase:
             # Get suggestions from hunspell dictionaries. But only
             # if input_phrase does not contain spaces. The hunspell
             # dictionaries contain only single words, not sentences.
@@ -484,8 +551,7 @@ class TabSqliteDb:
         # is not in the German hunspell dictionary as a single word but
         # created by suffix and prefix rules, the accent insensitive match
         # in the German hunspell dictionary would not find it either.
-        if input_phrase:
-            input_phrase = itb_util.remove_accents(input_phrase.lower())
+        input_phrase = itb_util.remove_accents(input_phrase.lower())
         # Now phrase_frequencies might contain something like this:
         #
         # {'code': 0, 'communicability': 0, 'cold': 0, 'colour': 0}
@@ -501,9 +567,8 @@ class TabSqliteDb:
         # But unfortunately that does not work when creating views,
         # (“OperationalError: parameters are not allowed in views”).
         quoted_input_phrase = ''
-        if input_phrase:
-            quoted_input_phrase = input_phrase.replace(
-                '\x00', '').replace('"', '""')
+        quoted_input_phrase = input_phrase.replace(
+            '\x00', '').replace('"', '""')
         self.database.execute('DROP VIEW IF EXISTS like_input_phrase_view;')
         sqlstr = f'''
         CREATE TEMPORARY VIEW IF NOT EXISTS like_input_phrase_view AS
@@ -681,23 +746,19 @@ class TabSqliteDb:
                 'Unexpected error adding description to user_db: %s: %s',
                  error.__class__.__name__, error)
 
-    def init_user_db(self) -> None:
-        '''
-        Initialize the user database unless it is an in-memory database
-        '''
-        if self.user_db_file == ':memory:':
-            return
-        if not os.path.exists(self.user_db_file):
-            self.database = sqlite3.connect(self.user_db_file)
-            self.set_database_pragma_options()
+    @classmethod
+    def sqlite3_connect_database(cls, db_file: str) -> sqlite3.Connection:
+        '''Connect to a database, creating it if it does not exist
 
-    def set_database_pragma_options(self) -> None:
-        '''Set options for the user database'''
-        # a database containing the complete German Hunspell
+        Also set our standard options for the database.
+        '''
+        # Creates the file db_file if it does not exist:
+        database_connection = sqlite3.connect(db_file)
+        # A database containing the complete German Hunspell
         # dictionary has less then 6000 pages. 20000 pages
         # should be enough to cache the complete database
         # in most cases.
-        self.database.executescript('''
+        database_connection.executescript('''
         PRAGMA encoding = "UTF-8";
         PRAGMA case_sensitive_like = true;
         PRAGMA page_size = 4096;
@@ -708,7 +769,11 @@ class TabSqliteDb:
         PRAGMA synchronous = NORMAL;
         PRAGMA auto_vacuum = FULL;
         ''')
-        self.database.commit()
+        database_connection.commit()
+        database_connection.text_factory = (
+            lambda x: x.decode(
+                encoding='utf-8', errors='replace')) # or better 'ignore'?
+        return database_connection
 
     @classmethod
     def get_database_desc(cls, db_file: str) -> Optional[Dict[str, str]]:
@@ -716,7 +781,7 @@ class TabSqliteDb:
         if not os.path.exists(db_file):
             return None
         try:
-            database = sqlite3.connect(db_file)
+            database = cls.sqlite3_connect_database(db_file)
             desc = {}
             for row in database.execute("SELECT * FROM desc;").fetchall():
                 desc[row[0]] = row[1]
@@ -749,7 +814,7 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         if not os.path.exists(db_file):
             return None
         try:
-            database = sqlite3.connect(db_file)
+            database = cls.sqlite3_connect_database(db_file)
             table_phrases_result = database.execute(
                 "select sql from sqlite_master where name='phrases';"
             ).fetchall()
@@ -987,6 +1052,30 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                         commit=commit)
         return
 
+    def phrase_exists(self, phrase: str) -> int:
+        '''
+        Checks if an entry for phrase already exists in the user database
+
+        :param phrase: The phrase to check whether it is already recorded
+        :return: 0 if not in the database. > 0 if in the database.
+                 The value > 0 gives the sum(user_freq) in the database
+                 for phrase.
+        '''
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug('phrase=%r', phrase)
+        if not phrase:
+            return 0
+        phrase = unicodedata.normalize(
+            itb_util.NORMALIZATION_FORM_INTERNAL, phrase)
+        select_sqlstr = '''
+        SELECT sum(user_freq) FROM user_db.phrases WHERE phrase = :phrase ;
+        '''
+        select_sqlargs = {'phrase': phrase}
+        retval = self.database.execute(select_sqlstr, select_sqlargs).fetchall()
+        if retval[0][0] is None:
+            return 0
+        return int(retval[0][0])
+
     def remove_phrase(
             self,
             input_phrase: str = '',
@@ -998,9 +1087,7 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         no matter for what input phrase from the database.
         '''
         if DEBUG_LEVEL > 1:
-            LOGGER.debug(
-                'TabSqliteDb.remove_phrase() phrase=%(p)s',
-                {'p': phrase.encode('UTF-8')})
+            LOGGER.debug('phrase=%r', phrase)
         if not phrase:
             return
         phrase = unicodedata.normalize(
@@ -1023,18 +1110,25 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         if commit:
             self.database.commit()
 
-    def extract_user_phrases(self) -> List[Tuple[str, int]]:
+    def _extract_user_phrases(self) -> List[Tuple[str, str, int]]:
         '''extract user phrases from database'''
+        LOGGER.info(
+            'Trying to recover phrases from old, incompatible database'
+            'Phrases successfully recovered can be used to '
+            'initialize a newly created database.')
         try:
-            database = sqlite3.connect(self.user_db_file)
+            database = self.sqlite3_connect_database(self.user_db_file)
             database.execute('PRAGMA wal_checkpoint;')
             phrases = database.execute(
-                'SELECT phrase, sum(user_freq) FROM phrases GROUP BY phrase;'
+                'SELECT input_phrase, phrase, sum(user_freq) FROM phrases GROUP BY phrase;'
             ).fetchall()
             database.close()
             phrases = [
                 (unicodedata.normalize(
-                    itb_util.NORMALIZATION_FORM_INTERNAL, x[0]), x[1])
+                    itb_util.NORMALIZATION_FORM_INTERNAL, x[0]),
+                 unicodedata.normalize(
+                    itb_util.NORMALIZATION_FORM_INTERNAL, x[1]),
+                 x[2])
                 for x in
                 phrases
             ]
@@ -1180,7 +1274,6 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         except Exception as error: # pylint: disable=broad-except
             LOGGER.exception('Unexpected error dumping database: %s: %s',
                               error.__class__.__name__, error)
-            return
 
     def number_of_rows_in_database(self) -> int:
         '''
@@ -1223,7 +1316,7 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                 # SQLite objects created in a thread can only be used in
                 # that same thread.  As the database cleanup is usually
                 # called in a separate thread, get a new connection:
-                database = sqlite3.connect(self.user_db_file)
+                database = self.sqlite3_connect_database(self.user_db_file)
             else:
                 database = self.database
             rows = database.execute("SELECT * FROM phrases;").fetchall()
